<?php declare(strict_types=1);

namespace App\Controller;

error_reporting(error_level: E_ALL);

use App\Repository\DomainRepository;
use App\Repository\DynDNSRepository;
use DI\Container;
use DI\ContainerBuilder;
use Monolog\Formatter\LineFormatter;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;
use function DI\autowire;

/**
 *
 */
class BindAPI
{
	private Logger $logger;
	private Container $container;
	
	
	/**
	 * @throws \Exception
	 */
	public function __construct($verbose = false)
	{
		// init the logger
		$dateFormat = "Y:m:d H:i:s";
		$output = "%datetime% %channel%.%level_name% %message%\n"; // %context% %extra%
		$formatter = new LineFormatter(format: $output, dateFormat: $dateFormat);
		
		$debug = (new ConfigController)->getConfig(configKey: 'debug');
		if ($debug) {
			$stream = new StreamHandler(stream: dirname(path: __DIR__, levels: 2) . '/bindAPI.log', level: Logger::DEBUG);
		} else {
			$stream = new StreamHandler(stream: dirname(path: __DIR__, levels: 2) . '/bindAPI.log', level: Logger::INFO);
		}
		$stream->setFormatter(formatter: $formatter);
		
		$this->logger = new Logger(name: 'bindAPI');
		$this->logger->pushHandler(handler: $stream);
		$this->logger->debug(message: 'bindAPI started');
		
		
		$containerBuilder = new ContainerBuilder();
		$containerBuilder->addDefinitions([
			ConfigController::class => autowire()
				->constructorParameter(parameter: 'verbose', value: $verbose),
			CLIController::class      => autowire()
				->constructorParameter(parameter: 'logger', value: $this->logger),
			DomainController::class   => autowire()
				->constructorParameter(parameter: 'logger', value: $this->logger),
			DomainRepository::class   => autowire()
				->constructorParameter(parameter: 'logger', value: $this->logger),
			DynDnsRepository::class  => autowire()
				->constructorParameter(parameter: 'logger', value: $this->logger),
			RequestController::class => autowire()
				->constructorParameter(parameter: 'logger', value: $this->logger)
		]);
		$this->container = $containerBuilder->build();
	}
	
	
	/**
	 * @throws \DI\DependencyException
	 * @throws \DI\NotFoundException
	 */
	public function runCommand(int $argumentsCount, array $arguments): void
	{
		$this->logger->debug(message: 'runCommand()');
		$cliController = $this->container->get(name: CLIController::class);
		$cliController->runCommand(argumentsCount: $argumentsCount, arguments: $arguments);
	}
	
	
	/**
	 * @throws \DI\DependencyException
	 * @throws \DI\NotFoundException
	 */
	public function handleRequest(string $requestMethod, array $uri): void
	{
		$this->logger->debug(message: 'handleRequest()');
		$requestController = $this->container->get(name: RequestController::class);
		$requestController->handleRequest(requestMethod: $requestMethod, uri: $uri);
	}
}
