<?php declare(strict_types=1);

namespace App\Repository;

use App\Controller\DatabaseConnection;
use App\Entity\Domain;
use App\Entity\Panel;
use PDO;
use PDOException;

/**
 *
 */
class PanelRepository
{
	public function __construct(private readonly DatabaseConnection $databaseConnection)
	{}
	
	public function findSelf(): array
	{
		$sql = "
			SELECT id, name, a, aaaa, apikey, self
            FROM " . DatabaseConnection::TABLE_PANELS . "
            WHERE self = 1";
		
		$panels = [];
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->execute();
			while ($result = $statement->fetch(mode: PDO::FETCH_ASSOC)) {
				$panel = new Panel(name: $result['name'], id: $result['id'], a: $result['a'], aaaa: $result['aaaa'], apikey: $result['apikey'], self: $result['self']);
				$panels[] = $panel;
			}
			return $panels;
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
		
		
	}
	
	/**
	 * @return array
	 */
	public function findAll(): array
	{
		$panels = [];
		$sql = "
            SELECT id, name, a, aaaa, apikey, self
            FROM " . DatabaseConnection::TABLE_PANELS . "
            ORDER BY name";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->execute();
			while ($result = $statement->fetch(mode: PDO::FETCH_ASSOC)) {
				$panel = new Panel(name: $result['name'], id: $result['id'], a: $result['a'], aaaa: $result['aaaa'], apikey: $result['apikey'], self: $result['self']);
				$panels[] = $panel;
			}
			return $panels;
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	/**
	 * @param int $id
	 *
	 * @return null|\App\Entity\Panel
	 */
	public function findByID(int $id): ?Panel
	{
		$sql = "
			SELECT id, name, a, aaaa, apikey, self
			FROM . " . DatabaseConnection::TABLE_PANELS . "
            WHERE id = :id";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: ':id', var: $id);
			$statement->execute();
			if ($result = $statement->fetch(mode: PDO::FETCH_ASSOC)) {
				return new Panel(name: $result['name'], a: $result['a'], aaaa: $result['aaaa'], apikey: $result['apikey'], self: $result['self']);
			} else {
				return null;
			}
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	/**
	 * @param String $name
	 *
	 * @return \App\Entity\Panel|bool
	 */
	public function findByName(string $name): Panel|bool
	{
		$sql = "
			SELECT id, name, a, aaaa, apikey, self
			FROM " . DatabaseConnection::TABLE_PANELS . "
            WHERE name = :name";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: ':name', var: $name);
			$statement->execute();
			if ($result = $statement->fetch(mode: PDO::FETCH_ASSOC)) {
				return new Panel(name: $result['name'], a: $result['a'], aaaa: $result['aaaa'], apikey: $result['apikey'], self: $result['self']);
			} else {
				return false;
			}
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	/**
	 * @param String $name
	 * @param String $a
	 * @param String $aaaa
	 * @param String $apikey
	 *
	 * @return string|false
	 */
	public function insert(string $name, string $a, string $aaaa, String $apikey, int $self): bool|string
	{
		$sql = "
            INSERT INTO " . DatabaseConnection::TABLE_PANELS . " (name, a, aaaa, apikey, self)
            VALUES (:name, :a, :aaaa, :apikey, :self)";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: ':name', var: $name);
			$statement->bindParam(param: ':a', var: $a);
			$statement->bindParam(param: ':aaaa', var: $aaaa);
			$statement->bindParam(param: ':apikey', var: $apikey);
			$statement->bindParam(param: ':self', var: $self);
			$statement->execute();
			
			return $this->databaseConnection->getConnection()->lastInsertId();
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	/**
	 * @param Int    $id
	 * @param String $name
	 * @param String $a
	 * @param String $aaaa
	 * @param String $apikey
	 *
	 * @return false|int
	 */
	public function update(int $id, string $name, string $a, string $aaaa, String $apikey, int $self): bool|int
	{
		$current = $this->findByID(id: $id);
		
		if (empty($name)) {
			$name = $current->getName();
		}
		if (empty($a)) {
			$a = $current->getA();
		}
		if (empty($aaaa)) {
			$aaaa = $current->getAaaa();
		}
		if (empty($apikey)) {
			$apikey = $current->getApikey();
		}
		
		if (empty($self)) {
			echo "self is empty";
			$self = $current->getSelf();
		} else {
			if ($self == -1) {
				$self = 0;
			}
		}
		
		$sql = "
			UPDATE " . DatabaseConnection::TABLE_PANELS . " SET
				name = :name,
				a = :a,
               	aaaa = :aaaa,
               	apikey = :apikey,
               	self = :self
            WHERE id = :id";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: 'id', var: $id);
			$statement->bindParam(param: 'name', var: $name);
			$statement->bindParam(param: 'a', var: $a);
			$statement->bindParam(param: 'aaaa', var: $aaaa);
			$statement->bindParam(param: 'apikey', var: $apikey);
			$statement->bindParam(param: 'self', var: $self);
			$statement->execute();
			
			return $statement->rowCount();
		} catch (PDOException $e) {
			echo $e->getMessage();
			return false;
		}
	}
	
	
	/**
	 * @param $id
	 *
	 * @return int
	 */
	public function delete($id): int
	{
		$sql = "
            DELETE FROM " . DatabaseConnection::TABLE_PANELS . "
            WHERE id = :id";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: 'id', var: $id);
			$statement->execute();
			
			return $statement->rowCount();
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	/**
	 * @param String $field
	 *
	 * @return int
	 */
	public function getLongestEntry(String $field): int
	{
		$sql = "
            SELECT MAX(LENGTH(" . $field . ")) as length FROM " . DatabaseConnection::TABLE_PANELS;
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->execute();
			$result =  $statement->fetch();
			return $result['length'];
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
}
